@@ -277,6 +277,7 @@ messages!(
     (ShareProjectResponse, Foreground),
     (ShowContacts, Foreground),
     (StartLanguageServer, Foreground),
+    (SubscribeToChannels, Foreground),
     (SynchronizeBuffers, Foreground),
     (SynchronizeBuffersResponse, Foreground),
     (TaskContextForLocation, Background),
@@ -331,6 +332,7 @@ messages!(
     (RegenerateDevServerTokenResponse, Foreground),
     (RenameDevServer, Foreground),
     (OpenNewBuffer, Foreground),
+    (RestartLanguageServers, Foreground),
 );
 
 request_messages!(
@@ -440,7 +442,8 @@ request_messages!(
     (DeleteDevServer, Ack),
     (DeleteDevServerProject, Ack),
     (RegenerateDevServerToken, RegenerateDevServerTokenResponse),
-    (RenameDevServer, Ack)
+    (RenameDevServer, Ack),
+    (RestartLanguageServers, Ack)
 );
 
 entity_messages!(
@@ -470,6 +473,7 @@ entity_messages!(
     JoinProject,
     LeaveProject,
     MultiLspQuery,
+    RestartLanguageServers,
     OnTypeFormatting,
     OpenNewBuffer,
     OpenBufferById,
